#!/usr/local/bin/python
#
#
#			   filter00.py  Version 3.0
#
#		    Copyright (c) 1999  ThePorchDotCom, LLC
#	       Original idea by Phillip Porch (ppp@theporch.com)
#	    Written by Stephen Modena, AB4EL (shimshon@theporch.com)
#			    All rights reserved.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by 
#    the Free Software Foundation; either version 2, or (at your option) any
#    later version, or
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See either
#    the GNU General Public License or the Artistic License for more details.
#
#    You should also have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software Foundation,
#    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
#    Contact information:
#    Phillip Porch (ppp@theporch.com)
#    Stephen Modena (shimshon@theporch.com)
# Original idea by Phillip P. Porch
# Code author Steve Modena, AB4EL
#

#*********************modification history********************
#  "filter00.py  Version 1.0" became "filter00.py  Version 2.0"
#		by change of message text in "def trashCanMsg()"
#		by streamlining to more defs
#*************************************************************
#  "filter00.py  Version 2.0" became "filter00.py  Version 3.0"
#		by addition of lastBoundary routines
#*************************************************************

#
#	filter00.py assumes that stdin and stdout are a pipeline
#
#	If you have a listproc mailing list called test ,for example, the
#	following would be placed in the aliases for sendmail for the list.
#  test: "| /usr/local/bin/filter00.py | /usr/server/catmail -L TEST -f"
#
#	If you want to limit messages by line count, search for the ##### in 
#	theprogram and uncomment the following line. You set the maximum
#	number of lines in a message with the MAX_MSG_LINE_COUNT = variable.

######################### 
#                       #  
#	PREAMBLE	#
#                       #
#########################

# ! ! ! ! !
# in case there is a need to limit length of message body
# ! ! ! ! !
MAX_MSG_LINE_COUNT = 200;

#	bring in needed modules
import sys 	# importing 'sys' gives access to stdin and stdout
import re	# import the RegularExpressions module

###########################################
#  start def's
###########################################
def messageTooLong(aWork):

	aWork.append("* * * * * * * * * * * * * * * * * * * *\n")	
	aWork.append("*  MESSAGE EXCEEDED PERMITTED LENGTH  *\n")	
	aWork.append("*                                     *\n")	
	aWork.append("*          message truncated          *\n")	
	aWork.append("*                                     *\n")	
	aWork.append("* * * * * * * * * * * * * * * * * * * *\n")
		
############################################	
def trashCanMsg(aWork):

	aWork.append("Content-Type: text/plain; charset=us-ascii\n");
	aWork.append("Content-Transfer-Encoding: 7bit\n");
	aWork.append("\n");
	aWork.append("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n")
	aWork.append("*         ---REMAINDER OF MESSAGE TRUNCATED---            *\n")
	aWork.append("*     This post contains a forbidden message format       *\n")
	aWork.append("*  (such as an attached file, a v-card, HTML formatting)  *\n")
	aWork.append("*    Mail Lists at theporch.com only accept PLAIN TEXT    *\n")
	aWork.append("* If your postings display this message your mail program *\n")
	aWork.append("* is not set to send PLAIN TEXT ONLY and needs adjusting  *\n")
	aWork.append("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n")
	
#############################################
# 	search for header line based on pattern
def findContentTypeIfItExists(anArray, aPattern):

	contentTypeStr = ''					# null string
	for aLine in anArray:					# step through header
	    if aPattern.match(aLine) :				# check for match
    		contentTypeStr = aPattern.match(aLine).group(1)	# found it, so remember it
		break						# ...so stop searching
	return contentTypeStr

#############################################
def getBoundaryIfItExists(anArray):

	PATTERNboundary0 = re.compile(r'Content-type:.*boundary *= *"(.*)"', re.I|re.DOTALL)
	theBoundary =  ''
	aHeaderAsString = ''
	for aLine in anArray: 
		aHeaderAsString = aHeaderAsString + aLine	# concatenate header

	mo = PATTERNboundary0.search(aHeaderAsString)		# search...not match
	if( mo ): theBoundary = mo.group(1)			# succeed? : grab within "(...)"
	return theBoundary

#############################################
#	read in just the email header, i.e. until first blank line
def readMailHeaderIntoArray(anArray):
	while TRUE:						# loop infinitely
		lineIn = sys.stdin.readline()			# read in line by line
		anArray.append(lineIn)			# add to work
		if len(lineIn[:-1]) == 0 : break		# found end of header? : break out of loop
	
#############################################
# 	continue reading the file in
def continueReadingTheFileIn(anArray):			# ref to existing array
	headerSize = len(anArray)
	for lineIn in sys.stdin.readlines():			# read while lines available
		anArray.append(lineIn)				# add to work			
		##### to limit email body length to MAX_MSG_LINE_COUNT, UNCOMMENT next line ####
		##### and keep indentation where it is! ####
		#if len(anArray)-headerSize >= MAX_MSG_LINE_COUNT : messageTooLong(anArray); break

#############################################
#  end def's
#############################################
# 	some useful definitions
TRUE = 1;
FALSE = 0;

#	pattern stuff

#	save input stream here
headerArray = []	# init to empty list (array)
workArray = []		# init to empty list (array)

#################################
#                               #
#	CODE RUN STARTS HERE	#
#                               #
#################################

# get email header
readMailHeaderIntoArray(headerArray)
headerSize = len(headerArray)
workArray = headerArray[:]		#note syntax for making a COPY not a reference

# look for content type
PATTERNcontentType=re.compile(r'^Content-Type: *(.*) *;', re.I)	# this is the pattern
contentTypeStr = findContentTypeIfItExists(workArray, PATTERNcontentType)

# 	continue reading the file in
continueReadingTheFileIn(workArray)

#	is this email entirely a "text/plain" MIME type?
PATTERNtextPlain = re.compile(r'text/plain', re.I)	# prepare a reg expression
if( PATTERNtextPlain.search(contentTypeStr) ):		# yes it is
	sys.stdout.writelines(workArray)    		# ...so send entire work to output

else:							# otherwise do some more processing

	PATTERNmessage = re.compile(r'message/', re.I)		# we need this pattern now also
	PATTERNdigest = re.compile(r'multipart/digest', re.I)	# we need this pattern now also
	indexer = headerSize-1					# pointer to where we are
	for aLine in workArray[headerSize:]:			# loop through body of email
		indexer = indexer + 1				# keep track of where we are
		if PATTERNcontentType.match(aLine) :			# if this a Content-Type: line?
			if PATTERNtextPlain.search(aLine) :		# is it text/plain ? OK do nada
				pass
			elif PATTERNmessage.search(aLine) : 		# is it message/ ? OK do nada
				pass
			elif PATTERNdigest.search(aLine) : 		# is it multipart/digest ? OK do nada
				pass
			else: 
				workArray = workArray[:indexer] 	# truncate
				trashCanMsg(workArray)			# append explanation
				workArray.append("\n")
				workArray.append("--"+getBoundaryIfItExists(headerArray)+"--\n")
				workArray.append("\n")
				workArray.append("\n")
				break 					# break out of loop
	
	sys.stdout.writelines(workArray)
#
#
#
# end of filter00.py
